@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
-"""`tldextract` accurately separates the gTLD or ccTLD (generic or country code
-top-level domain) from the registered domain and subdomains of a URL.
+""" `tldextract` accurately separates a URL's subdomain, domain, and public suffix,
+using the Public Suffix List (PSL).
 
     >>> import tldextract
 
@@ -49,41 +48,40 @@ or suffix were found:
     '127.0.0.1'
 """
 
-import collections
 import logging
 import os
+import re
 from functools import wraps
+from typing import FrozenSet, List, NamedTuple, Optional, Sequence, Union
 
 import idna
 
-from .cache import DiskCache
-from .remote import IP_RE
-from .remote import SCHEME_RE
-from .remote import looks_like_ip
+from .cache import DiskCache, get_cache_dir
+from .remote import IP_RE, SCHEME_RE, looks_like_ip
 from .suffix_list import get_suffix_lists
 
 LOG = logging.getLogger("tldextract")
 
-CACHE_DIR_DEFAULT = os.path.join(os.path.dirname(__file__), '.suffix_cache/')
-CACHE_DIR = os.path.expanduser(os.environ.get("TLDEXTRACT_CACHE", CACHE_DIR_DEFAULT))
-CACHE_TIMEOUT = os.environ.get('TLDEXTRACT_CACHE_TIMEOUT')
+
+CACHE_TIMEOUT = os.environ.get("TLDEXTRACT_CACHE_TIMEOUT")
 
 PUBLIC_SUFFIX_LIST_URLS = (
-    'https://publicsuffix.org/list/public_suffix_list.dat',
-    'https://raw.githubusercontent.com/publicsuffix/list/master/public_suffix_list.dat',
+    "https://publicsuffix.org/list/public_suffix_list.dat",
+    "https://raw.githubusercontent.com/publicsuffix/list/master/public_suffix_list.dat",
 )
 
-CACHE = DiskCache(cache_dir=CACHE_DIR)
+_UNICODE_DOTS_RE = re.compile("[\u002e\u3002\uff0e\uff61]")
 
 
-class ExtractResult(collections.namedtuple('ExtractResult', 'subdomain domain suffix')):
-    '''namedtuple of a URL's subdomain, domain, and suffix.'''
+class ExtractResult(NamedTuple):
+    """namedtuple of a URL's subdomain, domain, and suffix."""
 
-    # Necessary for __dict__ member to get populated in Python 3+
-    __slots__ = ()
+    subdomain: str
+    domain: str
+    suffix: str
 
     @property
-    def registered_domain(self):
+    def registered_domain(self) -> str:
         """
         Joins the domain and suffix fields with a dot, if they're both set.
 
@@ -93,11 +91,11 @@ class ExtractResult(collections.namedtuple('ExtractResult', 'subdomain domain su
         ''
         """
         if self.domain and self.suffix:
-            return self.domain + '.' + self.suffix
-        return ''
+            return self.domain + "." + self.suffix
+        return ""
 
     @property
-    def fqdn(self):
+    def fqdn(self) -> str:
         """
         Returns a Fully Qualified Domain Name, if there is a proper domain/suffix.
 
@@ -107,12 +105,13 @@ class ExtractResult(collections.namedtuple('ExtractResult', 'subdomain domain su
         ''
         """
         if self.domain and self.suffix:
-            # self is the namedtuple (subdomain domain suffix)
-            return '.'.join(i for i in self if i)
-        return ''
+            # Disable bogus lint error (https://github.com/PyCQA/pylint/issues/2568)
+            # pylint: disable-next=not-an-iterable
+            return ".".join(i for i in self if i)
+        return ""
 
     @property
-    def ipv4(self):
+    def ipv4(self) -> str:
         """
         Returns the ipv4 if that is what the presented domain/url is
 
@@ -125,29 +124,34 @@ class ExtractResult(collections.namedtuple('ExtractResult', 'subdomain domain su
         """
         if not (self.suffix or self.subdomain) and IP_RE.match(self.domain):
             return self.domain
-        return ''
+        return ""
 
 
-class TLDExtract(object):
-    '''A callable for extracting, subdomain, domain, and suffix components from
-    a URL.'''
+class TLDExtract:
+    """A callable for extracting, subdomain, domain, and suffix components from
+    a URL."""
 
     # TODO: Agreed with Pylint: too-many-arguments
-    def __init__(self, cache_dir=CACHE_DIR, suffix_list_urls=PUBLIC_SUFFIX_LIST_URLS,  # pylint: disable=too-many-arguments
-                 fallback_to_snapshot=True, include_psl_private_domains=False, extra_suffixes=(),
-                 cache_fetch_timeout=CACHE_TIMEOUT):
+    def __init__(  # pylint: disable=too-many-arguments
+        self,
+        cache_dir: Optional[str] = get_cache_dir(),
+        suffix_list_urls: Sequence[str] = PUBLIC_SUFFIX_LIST_URLS,
+        fallback_to_snapshot: bool = True,
+        include_psl_private_domains: bool = False,
+        extra_suffixes: Sequence[str] = (),
+        cache_fetch_timeout: Union[str, float, None] = CACHE_TIMEOUT,
+    ) -> None:
         """
         Constructs a callable for extracting subdomain, domain, and suffix
         components from a URL.
 
-        Upon calling it, it first checks for a JSON in `cache_dir`.
-        By default, the `cache_dir` will live in the tldextract directory.
-
-        You can disable the caching functionality of this module  by setting `cache_dir` to False.
+        Upon calling it, it first checks for a JSON in `cache_dir`. By default,
+        the `cache_dir` will live in the tldextract directory. You can disable
+        the caching functionality of this module by setting `cache_dir` to `None`.
 
         If the cached version does not exist (such as on the first run), HTTP request the URLs in
         `suffix_list_urls` in order, until one returns public suffix list data. To disable HTTP
-        requests, set this to something falsy.
+        requests, set this to an empty sequence.
 
         The default list of URLs point to the latest version of the Mozilla Public Suffix List and
         its mirror, but any similar document could be specified. Local files can be specified by
@@ -178,24 +182,32 @@ class TLDExtract(object):
         and read timeouts
         """
         suffix_list_urls = suffix_list_urls or ()
-        self.suffix_list_urls = tuple(url.strip() for url in suffix_list_urls if url.strip())
+        self.suffix_list_urls = tuple(
+            url.strip() for url in suffix_list_urls if url.strip()
+        )
 
         self.fallback_to_snapshot = fallback_to_snapshot
         if not (self.suffix_list_urls or cache_dir or self.fallback_to_snapshot):
-            raise ValueError("The arguments you have provided disable all ways for tldextract "
-                             "to obtain data. Please provide a suffix list data, a cache_dir, "
-                             "or set `fallback_to_snapshot` to `True`.")
+            raise ValueError(
+                "The arguments you have provided disable all ways for tldextract "
+                "to obtain data. Please provide a suffix list data, a cache_dir, "
+                "or set `fallback_to_snapshot` to `True`."
+            )
 
         self.include_psl_private_domains = include_psl_private_domains
         self.extra_suffixes = extra_suffixes
-        self._extractor = None
+        self._extractor: Optional[_PublicSuffixListTLDExtractor] = None
 
-        self.cache_fetch_timeout = cache_fetch_timeout
+        self.cache_fetch_timeout = (
+            float(cache_fetch_timeout)
+            if isinstance(cache_fetch_timeout, str)
+            else cache_fetch_timeout
+        )
         self._cache = DiskCache(cache_dir)
-        if isinstance(self.cache_fetch_timeout, str):
-            self.cache_fetch_timeout = float(self.cache_fetch_timeout)
 
-    def __call__(self, url, include_psl_private_domains=None):
+    def __call__(
+        self, url: str, include_psl_private_domains: Optional[bool] = None
+    ) -> ExtractResult:
         """
         Takes a string URL and splits it into its subdomain, domain, and
         suffix (effective TLD, gTLD, ccTLD, etc.) component.
@@ -207,39 +219,41 @@ class TLDExtract(object):
         ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk')
         """
 
-        netloc = SCHEME_RE.sub("", url) \
-            .partition("/")[0] \
-            .partition("?")[0] \
-            .partition("#")[0] \
-            .split("@")[-1] \
-            .partition(":")[0] \
-            .strip() \
+        netloc = (
+            SCHEME_RE.sub("", url)
+            .partition("/")[0]
+            .partition("?")[0]
+            .partition("#")[0]
+            .split("@")[-1]
+            .partition(":")[0]
+            .strip()
             .rstrip(".")
+        )
 
-        labels = netloc.split(".")
+        labels = _UNICODE_DOTS_RE.split(netloc)
 
         translations = [_decode_punycode(label) for label in labels]
         suffix_index = self._get_tld_extractor().suffix_index(
-            translations,
-            include_psl_private_domains=include_psl_private_domains
+            translations, include_psl_private_domains=include_psl_private_domains
         )
 
         suffix = ".".join(labels[suffix_index:])
         if not suffix and netloc and looks_like_ip(netloc):
-            return ExtractResult('', netloc, '')
+            return ExtractResult("", netloc, "")
 
-        subdomain = ".".join(labels[:suffix_index - 1]) if suffix_index else ""
+        subdomain = ".".join(labels[: suffix_index - 1]) if suffix_index else ""
         domain = labels[suffix_index - 1] if suffix_index else ""
         return ExtractResult(subdomain, domain, suffix)
 
-    def update(self, fetch_now=False):
+    def update(self, fetch_now: bool = False) -> None:
+        """Force fetch the latest suffix list definitions."""
         self._extractor = None
         self._cache.clear()
         if fetch_now:
             self._get_tld_extractor()
 
     @property
-    def tlds(self):
+    def tlds(self) -> List[str]:
         """
         Returns the list of tld's used by default
 
@@ -247,16 +261,15 @@ class TLDExtract(object):
         """
         return list(self._get_tld_extractor().tlds())
 
-    def _get_tld_extractor(self):
-        '''Get or compute this object's TLDExtractor. Looks up the TLDExtractor
+    def _get_tld_extractor(self) -> "_PublicSuffixListTLDExtractor":
+        """Get or compute this object's TLDExtractor. Looks up the TLDExtractor
         in roughly the following order, based on the settings passed to
         __init__:
 
         1. Memoized on `self`
         2. Local system _cache file
         3. Remote PSL, over HTTP
-        4. Bundled PSL snapshot file'''
-        # pylint: disable=no-else-return
+        4. Bundled PSL snapshot file"""
 
         if self._extractor:
             return self._extractor
@@ -265,7 +278,7 @@ class TLDExtract(object):
             cache=self._cache,
             urls=self.suffix_list_urls,
             cache_fetch_timeout=self.cache_fetch_timeout,
-            fallback_to_snapshot=self.fallback_to_snapshot
+            fallback_to_snapshot=self.fallback_to_snapshot,
         )
 
         if not any([public_tlds, private_tlds, self.extra_suffixes]):
@@ -275,7 +288,7 @@ class TLDExtract(object):
             public_tlds=public_tlds,
             private_tlds=private_tlds,
             extra_tlds=list(self.extra_suffixes),
-            include_psl_private_domains=self.include_psl_private_domains
+            include_psl_private_domains=self.include_psl_private_domains,
         )
         return self._extractor
 
@@ -284,12 +297,15 @@ TLD_EXTRACTOR = TLDExtract()
 
 
 @wraps(TLD_EXTRACTOR.__call__)
-def extract(url, include_psl_private_domains=False):
+def extract(  # pylint: disable=missing-function-docstring
+    url: str, include_psl_private_domains: Optional[bool] = False
+) -> ExtractResult:
     return TLD_EXTRACTOR(url, include_psl_private_domains=include_psl_private_domains)
 
 
 @wraps(TLD_EXTRACTOR.update)
-def update(*args, **kwargs):
+# pylint: disable-next=missing-function-docstring
+def update(*args, **kwargs):  # type: ignore[no-untyped-def]
     return TLD_EXTRACTOR.update(*args, **kwargs)
 
 
@@ -298,7 +314,13 @@ class _PublicSuffixListTLDExtractor:
     lookups.
     """
 
-    def __init__(self, public_tlds, private_tlds, extra_tlds, include_psl_private_domains=False):
+    def __init__(
+        self,
+        public_tlds: List[str],
+        private_tlds: List[str],
+        extra_tlds: List[str],
+        include_psl_private_domains: bool = False,
+    ):
         # set the default value
         self.include_psl_private_domains = include_psl_private_domains
         self.public_tlds = public_tlds
@@ -306,40 +328,49 @@ class _PublicSuffixListTLDExtractor:
         self.tlds_incl_private = frozenset(public_tlds + private_tlds + extra_tlds)
         self.tlds_excl_private = frozenset(public_tlds + extra_tlds)
 
-    def tlds(self, include_psl_private_domains=None):
+    def tlds(
+        self, include_psl_private_domains: Optional[bool] = None
+    ) -> FrozenSet[str]:
+        """Get the currently filtered list of suffixes."""
         if include_psl_private_domains is None:
             include_psl_private_domains = self.include_psl_private_domains
 
-        return self.tlds_incl_private if include_psl_private_domains else self.tlds_excl_private
+        return (
+            self.tlds_incl_private
+            if include_psl_private_domains
+            else self.tlds_excl_private
+        )
 
-    def suffix_index(self, lower_spl, include_psl_private_domains=None):
+    def suffix_index(
+        self, lower_spl: List[str], include_psl_private_domains: Optional[bool] = None
+    ) -> int:
         """Returns the index of the first suffix label.
         Returns len(spl) if no suffix is found
         """
         tlds = self.tlds(include_psl_private_domains)
         length = len(lower_spl)
         for i in range(length):
-            maybe_tld = '.'.join(lower_spl[i:])
-            exception_tld = '!' + maybe_tld
+            maybe_tld = ".".join(lower_spl[i:])
+            exception_tld = "!" + maybe_tld
             if exception_tld in tlds:
                 return i + 1
 
             if maybe_tld in tlds:
                 return i
 
-            wildcard_tld = '*.' + '.'.join(lower_spl[i + 1:])
+            wildcard_tld = "*." + ".".join(lower_spl[i + 1 :])
             if wildcard_tld in tlds:
                 return i
 
         return length
 
 
-def _decode_punycode(label):
+def _decode_punycode(label: str) -> str:
     lowered = label.lower()
-    looks_like_puny = lowered.startswith('xn--')
+    looks_like_puny = lowered.startswith("xn--")
     if looks_like_puny:
         try:
-            return idna.decode(label.encode('ascii')).lower()
+            return idna.decode(lowered)
         except (UnicodeError, IndexError):
             pass
     return lowered
